@@ -32,6 +32,7 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
+#include "panel-icon-names.h"
 #include "panel-stock-icons.h"
 
 GdkFilterReturn popup_filter (GdkXEvent *gdk_xevent,
@@ -66,7 +67,7 @@ display_popup_window (GdkScreen *screen)
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	gtk_widget_show (vbox);
 
-	image = gtk_image_new_from_icon_name (PANEL_FORCE_QUIT_ICON,
+	image = gtk_image_new_from_icon_name (PANEL_ICON_FORCE_QUIT,
 					      GTK_ICON_SIZE_DIALOG);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
 	gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 4);
@@ -144,8 +145,8 @@ find_managed_window (Window window)
 	Window  parent;
 	Window *kids = NULL;
 	Window  retval;
-	int     nkids, i;
-	int     result;
+	guint   nkids;
+	int     i, result;
 
 	if (wm_state_set (window))
 		return window;
@@ -174,18 +175,38 @@ find_managed_window (Window window)
 	return retval;
 }
 
+static void
+kill_window_response (GtkDialog *dialog,
+		      gint       response_id,
+		      gpointer   user_data)
+{
+	if (response_id == GTK_RESPONSE_ACCEPT) {
+		Window window = (Window) user_data;
+
+		gdk_error_trap_push ();
+		XKillClient (gdk_display, window);
+		gdk_flush ();
+		gdk_error_trap_pop ();
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 /* From metacity */
-static gboolean
-kill_window_question (void)
+static void
+kill_window_question (gpointer window)
 {
 	GtkWidget *dialog;
-	gboolean   retval;
  
 	dialog = gtk_message_dialog_new (NULL, 0,
-					 GTK_MESSAGE_QUESTION,
+					 GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_NONE,
-					 _("Force this application to exit?\n"
-					   "(Any open documents will be lost.)"));
+					 _("Force this application to exit?"));
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("If you choose to force an application "
+						  "to exit, any open documents in it will "
+						  "be lost."));
  
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL,
@@ -194,13 +215,15 @@ kill_window_question (void)
 				GTK_RESPONSE_ACCEPT,
 				NULL);
  
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CANCEL);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Force Quit"));
 
-	retval = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT;
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (kill_window_response), window);
 
-	gtk_widget_destroy (dialog);
-
-	return retval;
+	gtk_widget_show (dialog);
 }
 
 static void 
@@ -219,8 +242,10 @@ handle_button_press_event (GtkWidget *popup,
 
 	window = find_managed_window (event->subwindow);
 
-	if (window != None && kill_window_question ())
-		XKillClient (gdk_display, window);
+	if (window != None) {
+		if (!gdk_xid_table_lookup_for_display (gdk_display_get_default (), window))
+			kill_window_question ((gpointer) window);
+	}
 }
 
 GdkFilterReturn
@@ -248,7 +273,8 @@ popup_filter (GdkXEvent *gdk_xevent,
 }
 
 void
-panel_force_quit (GdkScreen *screen)
+panel_force_quit (GdkScreen *screen,
+		  guint      time)
 {
 	GdkGrabStatus  status;
 	GdkCursor     *cross;
@@ -263,7 +289,7 @@ panel_force_quit (GdkScreen *screen)
 
 	cross = gdk_cursor_new (GDK_CROSS);
 	status = gdk_pointer_grab (root, FALSE, GDK_BUTTON_PRESS_MASK,
-				   NULL, cross, GDK_CURRENT_TIME);
+				   NULL, cross, time);
 	gdk_cursor_unref (cross);
 	if (status != GDK_GRAB_SUCCESS) {
 		g_warning ("Pointer grab failed\n");
@@ -271,7 +297,7 @@ panel_force_quit (GdkScreen *screen)
 		return;
 	}
 
-	status = gdk_keyboard_grab (root, FALSE, GDK_CURRENT_TIME);
+	status = gdk_keyboard_grab (root, FALSE, time);
 	if (status != GDK_GRAB_SUCCESS) {
 		g_warning ("Keyboard grab failed\n");
 		remove_popup (popup);
